@@ -37,17 +37,32 @@ class BiasQuoteElement(PairwiseElement):
 
     @staticmethod
     def add_command_line_parameters(parser: argparse.ArgumentParser) -> None:
-        parser.add_argument("--biasquote-factor", type=Decimal, action="append",
-                            help="bias factor to apply to quotes. Prices will be multiplied by this factor, so a number less than 1 will reduce prices and a number greater than 1 will increase prices. For example, use 1.001 to increase prices by 10 bips. Can be specified multiple times to apply to different levels.")
+        parser.add_argument(
+            "--biasquote-factor",
+            type=Decimal,
+            action="append",
+            help="bias factor to apply to quotes. Prices will be multiplied by this factor, so a number less than 1 will reduce prices and a number greater than 1 will increase prices. For example, use 1.001 to increase prices by 10 bips. Can be specified multiple times to apply to different levels.",
+        )
 
     @staticmethod
     def from_command_line_parameters(args: argparse.Namespace) -> "BiasQuoteElement":
         bias_factors: typing.Sequence[Decimal] = args.biasquote_factor
         return BiasQuoteElement(bias_factors or [Decimal(1)])
 
-    def process_order_pair(self, context: mango.Context, model_state: ModelState, index: int, buy: typing.Optional[mango.Order], sell: typing.Optional[mango.Order]) -> typing.Tuple[typing.Optional[mango.Order], typing.Optional[mango.Order]]:
+    def process_order_pair(
+        self,
+        context: mango.Context,
+        model_state: ModelState,
+        index: int,
+        buy: typing.Optional[mango.Order],
+        sell: typing.Optional[mango.Order],
+    ) -> typing.Tuple[typing.Optional[mango.Order], typing.Optional[mango.Order]]:
         # If no bias is explicitly specified for this element, just use the last specified bias.
-        bias_factor: Decimal = self.bias_factors[index] if index < len(self.bias_factors) else self.bias_factors[-1]
+        bias_factor: Decimal = (
+            self.bias_factors[index]
+            if index < len(self.bias_factors)
+            else self.bias_factors[-1]
+        )
         bias_description = "BUY more" if bias_factor > 1 else "SELL more"
 
         if bias_factor == 1:
@@ -58,17 +73,21 @@ class BiasQuoteElement(PairwiseElement):
         new_sell: typing.Optional[mango.Order] = None
         if buy is not None:
             new_buy_price: Decimal = buy.price * bias_factor
-            new_buy = buy.with_price(new_buy_price)
-            self._logger.debug(f"""Order change - bias factor of {bias_factor} shifted price to {bias_description}:
+            new_buy = buy.with_update(price=new_buy_price)
+            self._logger.debug(
+                f"""Order change - bias factor of {bias_factor} shifted price to {bias_description}:
     Old: {buy}
-    New: {new_buy}""")
+    New: {new_buy}"""
+            )
 
         if sell is not None:
             new_sell_price: Decimal = sell.price * bias_factor
-            new_sell = sell.with_price(new_sell_price)
-            self._logger.debug(f"""Order change - bias factor of {bias_factor} shifted price to {bias_description}:
+            new_sell = sell.with_update(price=new_sell_price)
+            self._logger.debug(
+                f"""Order change - bias factor of {bias_factor} shifted price to {bias_description}:
     Old: {sell}
-    New: {new_sell}""")
+    New: {new_sell}"""
+            )
 
         return new_buy, new_sell
 
